class InvalidWhen(Exception): pass


class InvalidPeriodWhen(Exception): pass


class InvalidMatch(Exception): pass


class InvalidPeriod(Exception): pass


class AlreadyRegistered(Exception): pass


class RuleNotRegistered(Exception): pass
